#ifndef Color_H__
#define Color_H__

#include <inttypes.h>

// Low brightness values can create "stairstepping" effects if they are tween up and down.
// This is uncomfortable to watch when an object is in motion (i.e.: the Waves animation.)
// Workaround: when an RGB component is below LOW_STAIRSTEP, round it to 0 or LOW_STAIRSTEP.
// ...And yet!: If this number is too high, then there may be heavy contrast between the dark
// components and the lit ones.
#define LOW_STAIRSTEP  (0.26)

#include "Const.h"

// RGB: Range of each channel is 0.0...1.0
struct RGB {
  float r;  // red
  float g;  // green 
  float b;  // blue
};

inline RGB operator+( const RGB &c0, const RGB &c1 ) {
  return (RGB){ c0.r+c1.r, c0.g+c1.g, c0.b+c1.b };
};

inline void operator+=( RGB &c0, const RGB &c1 ) {
  c0.r += c1.r;
  c0.g += c1.g;
  c0.b += c1.b;
};

// Ranges: 0.0...1.0
struct HSV {
  float h;  // hue. Range: 0..1
  float s;  // saturation
  float v;  // value (brightness)
};

// ColorTween's should occur in HSV space so they look purty
struct ColorTween {
  float fromH;
  float fromS;
  float fromV;
  
  // Instead of tweening to a destination H, we'll tween either positively or negatively towards the _closest_ wrapped H value.
  // So red (H==0) -> magenta (H==0.99) doesn't have to tween through green (H==0.5)
  float addH;  
  float addS;
  float addV;
  
  float progress;
  float progressInc;
  
  bool isComplete;
};

class Color {
  public:
    //static RGB rgbFromUints( uint8_t, uint8_t, uint8_t );
    
    static uint8_t gammaValue( uint8_t );
    
    static RGB hsv2rgb( float, float, float );
    static HSV rgb2hsv( float, float, float );
    
    static void startColorTweenUsingHSV( ColorTween*, HSV, HSV, float );
    //static void startColorTweenUsingRGB( ColorTween*, RGB, RGB );
    static HSV colorTweenTickReturnHSV( ColorTween* );
    //static RGB colorTweenTickReturnRGB( ColorTween* );
    
    static RGB lerpRGB( RGB c0, RGB c1, float progress );
    static HSV lerpHSV( HSV c0, HSV c1, float progress );
};

#endif

